﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WpfApplication1
{
    using Infragistics.Windows.DataPresenter;
    using System.Windows;
    using Infragistics.Controls.Charts;
    using System.Collections;

    public static class CXamDataChartProgressiveItemsSource
    {
        #region ItemsSource
        public static DependencyProperty ItemsSourceProperty = DependencyProperty.RegisterAttached(
             "ItemsSource", 
             typeof(IEnumerable), 
             typeof(CXamDataChartProgressiveItemsSource),
             new PropertyMetadata(null, HandleItemsSourceChanged)
             );
        public static void SetItemsSource(HighDensityScatterSeries aHighDensityScatterSeries, IEnumerable aItemsSource)
        {
            aHighDensityScatterSeries.SetValue(ItemsSourceProperty, aItemsSource);
        }
        public static IEnumerable GetItemsSource(HighDensityScatterSeries aHighDensityScatterSeries)
        {
            return (IEnumerable) aHighDensityScatterSeries.GetValue(ItemsSourceProperty);
        }
        private static void HandleItemsSourceChanged(DependencyObject aObj, DependencyPropertyChangedEventArgs aArgs)
        {
            var aHighDensityScatterSeries = (HighDensityScatterSeries)aObj;
            var aProgressiveLoadIsPending = GetProgressiveLoadPending(aHighDensityScatterSeries);
            if (!aProgressiveLoadIsPending)
            {
                UpdateHighDensityScatterSeriesItemsSourceOnDemand(aHighDensityScatterSeries);
            }
        }
        #endregion
        #region ProgressiveLoadPending
        private static DependencyProperty ProgressiveLoadPendingProperty = DependencyProperty.RegisterAttached(
             "ProgressiveLoadPending",
             typeof(bool),
             typeof(CXamDataChartProgressiveItemsSource),
             new PropertyMetadata(false, HandleProgressiveLoadPendingChanged)
             );
        private static void SetProgressiveLoadPending(HighDensityScatterSeries aHighDensityScatterSeries, bool aProgressiveLoadPending)
        {
            aHighDensityScatterSeries.SetValue(ProgressiveLoadPendingProperty, aProgressiveLoadPending);
        }
        private static bool GetProgressiveLoadPending(HighDensityScatterSeries aHighDensityScatterSeries)
        {
            return (bool)aHighDensityScatterSeries.GetValue(ProgressiveLoadPendingProperty);
        }
        private static void HandleProgressiveLoadPendingChanged(DependencyObject aObj, DependencyPropertyChangedEventArgs aArgs)
        {
            var aProgressiveLoadIsPending = (bool)aArgs.NewValue;
            if (!aProgressiveLoadIsPending)
            {
                var aHighDensityScatterSeries = (HighDensityScatterSeries)aObj;
                UpdateHighDensityScatterSeriesItemsSourceOnDemand(aHighDensityScatterSeries);
            }
        }
        #endregion
        #region Active
        public static DependencyProperty ActiveProperty = DependencyProperty.RegisterAttached(
             "Active",
             typeof(bool),
             typeof(CXamDataChartProgressiveItemsSource),
             new PropertyMetadata(false, HandleActiveChanged)
             );
        public static void SetActive(HighDensityScatterSeries aHighDensityScatterSeries, bool aActive)
        {
            aHighDensityScatterSeries.SetValue(ActiveProperty, aActive);
        }
        public static bool GetActive(HighDensityScatterSeries aHighDensityScatterSeries)
        {
            return (bool)aHighDensityScatterSeries.GetValue(ActiveProperty);
        }
        private static void HandleActiveChanged(DependencyObject aObj, DependencyPropertyChangedEventArgs aArgs)
        {
            var aOldValue = (bool)aArgs.OldValue;
            var aNewValue = (bool)aArgs.NewValue;
            var aHighDensityScatterSeries = (HighDensityScatterSeries)aObj;
            var aProgressiveLoadStatusHandler = new EventHandler<ProgressiveLoadStatusEventArgs> (HandleProgressiveLoadStatus);
            if (aOldValue)
            {
                aHighDensityScatterSeries.ProgressiveLoadStatusChanged -= aProgressiveLoadStatusHandler;
            }
            if (aNewValue)
            {
                aHighDensityScatterSeries.ProgressiveLoadStatusChanged += aProgressiveLoadStatusHandler;
            }
        }
        #endregion
        #region ProgressiveLoadStatus
        private static void HandleProgressiveLoadStatus(object aSender, ProgressiveLoadStatusEventArgs aArgs)
        {
            var aProgressiveLoadIsPending = aArgs.CurrentStatus != 100;
            var aHighDensityScatterSeries = (HighDensityScatterSeries)aSender;
            SetProgressiveLoadPending(aHighDensityScatterSeries, aProgressiveLoadIsPending);
        }
        private static void UpdateHighDensityScatterSeriesItemsSourceOnDemand(HighDensityScatterSeries aHighDensityScatterSeries)
        {
            var aNewItemsSource = GetItemsSource(aHighDensityScatterSeries);
            var aCurrentItemsSource = aHighDensityScatterSeries.ItemsSource;
            if (aNewItemsSource != aCurrentItemsSource)
            {
                aHighDensityScatterSeries.ItemsSource = aNewItemsSource;
            }
        }
        #endregion
    }
}
